`define AC_ADDER_BUS_WITH 8
`define AC_ADDER_MAXIMUM_SUM_WIDTH 10

`define RESET 				1
`define IDLE 				2
`define COLLECT_DATA		3
`define GENERATE_OUTPUT 4

module adder8(
    output [`AC_ADDER_BUS_WITH - 1:0] data_out,
	 output valid_out,
	 output sot_out,
	 output eot_out,
	 output error,
	 
	 input [`AC_ADDER_BUS_WITH - 1:0] data_in,
	 input valid_in,
	 input sot_in,
	 input eot_in,
	 
	 input clk,
	 input rst_n
    );
	
	// State registers
	reg[3:0] state;
	reg[3:0] next_state;
	reg[`AC_ADDER_MAXIMUM_SUM_WIDTH : 0] next_acc_value;
	
	
	
	reg[`AC_ADDER_MAXIMUM_SUM_WIDTH - 1 : 0] accumulator;
	reg overflow;
	
	// OUTPUT registers
	reg [`AC_ADDER_BUS_WITH - 1:0] data_out_reg;
	reg valid_out_reg;
	reg sot_out_reg;
	reg eot_out_reg;
	reg error_reg;	
	
	reg [`AC_ADDER_BUS_WITH - 1:0] data1, data2;
	
	initial begin 
		next_state 	= `RESET;
		state 		= `RESET;
	end
	
	
	always @ (posedge clk)
	begin
		state 		<= next_state;
		accumulator <= next_acc_value;
		
		if(rst_n == 0)
			state <= `RESET;
	end
	
	always @ (*)
	begin
		data_out_reg = 0;
		valid_out_reg = 0;
		sot_out_reg = 0;
		eot_out_reg = 0;
		error_reg = 0;
		
		case (state)
		`RESET: begin
			next_acc_value = 0;  // THIS WAS ADDED
			accumulator = 0; 		// THIS WAS ADDED
			overflow = 0;
			next_state = `IDLE;
		end
		`IDLE:begin
			overflow = 0;
			next_acc_value = 0;  // THIS WAS ADDED
			accumulator = 0; 		// THIS WAS ADDED
			if(valid_in == 1 && sot_in == 1)
			begin
				next_acc_value = accumulator + data_in;
				next_state = /*`COLLECT_DATA; */(eot_in == 1) ? `GENERATE_OUTPUT : `COLLECT_DATA;
			end
		end
		`COLLECT_DATA: begin
			if(valid_in == 1) begin
				if (overflow == 0)
					next_acc_value = accumulator + data_in;
				
				if(next_acc_value > {`AC_ADDER_MAXIMUM_SUM_WIDTH{1'b1}})
				begin
					next_acc_value = {`AC_ADDER_MAXIMUM_SUM_WIDTH{1'b1}}; /*0;*/
					overflow = 1;
				end
			end
			if(eot_in == 1)
				next_state = `GENERATE_OUTPUT;
		end
		
		`GENERATE_OUTPUT: begin
			{data1, data2} = accumulator;
			
			error_reg = overflow;
			valid_out_reg = 1;
			sot_out_reg = 1;
		
			if (data1 == 0)
			begin
				eot_out_reg = 1;
				data_out_reg = data2;
				next_state = /*`RESET;*/ `IDLE;
			end
			else
			begin
				data_out_reg = data1;
				next_state = `GENERATE_OUTPUT + 1;
			end
			
						
		end
		
		`GENERATE_OUTPUT + 1: begin
			valid_out_reg = 1; // THIS WAS ADDED
			eot_out_reg = 1;   // THIS WAS ADDED
			error_reg = overflow;
			data_out_reg = data2;
			next_state = `IDLE;
		end
		
		endcase
	end
	
	assign data_out 	= data_out_reg;
	assign valid_out 	= valid_out_reg;
	assign sot_out 	= sot_out_reg ;
	assign eot_out 	= eot_out_reg ;
	assign error 		= error_reg;
	
endmodule
