`timescale 1ns / 1ps

////////////////////////////////////////////////////////////////////////////////
// Company: 
// Engineer:
//
// Create Date:   16:49:47 10/05/2019
// Design Name:   average
// Module Name:   average_test.v
// Project Name:  ex1_skel
// Target Device:  
// Tool versions:  
// Description: 
//
// Verilog Test Fixture created by ISE for module: average
//
// Dependencies:
// 
// Revision:
// Revision 0.01 - File Created
// Additional Comments:
// 
////////////////////////////////////////////////////////////////////////////////

`define AC_ADDER_BUS_WITH 8
`define AC_ADDER_MAXIMUM_SUM_WIDTH 10


module adder8_test;

	// Inputs
	wire [`AC_ADDER_BUS_WITH - 1:0] data_in;
	wire valid_in;
	wire sot_in;
	wire eot_in;
	
	// Control regs
	reg [`AC_ADDER_BUS_WITH - 1:0] data_in_reg;
	reg valid_in_reg;
	reg sot_in_reg;
	reg eot_in_reg;
	

	// Outputs
	wire [`AC_ADDER_BUS_WITH - 1:0] data_out;
	wire valid_out;
	wire sot_out;
	wire eot_out;
	wire error;	
	
	// Clock and reset
	reg clk, rst_n;
	
	integer sum = 0;
	reg error_reg;
	
	//Veriication stuff
	
	// Random interator
	integer i;
	integer test_index = 0;
	reg expecting_eot;
	
	//Test vectors
	reg[399:0] test0 = {8'd2, 8'd8, 8'd24, 8'd67, 8'd55, 8'd93, 8'd6}; // normal
	reg[399:0] test1 = {8'd1, 8'd2, 8'd3, 8'd4, 8'd35, 8'd5, 8'd6, 8'd7, 8'd8}; // normal
	reg[399:0] test2 = {8'h15, 8'hA0, 8'h35, 8'hFD, 8'h68}; // 10 bit output
	reg[399:0] test3 = {8'd4, 8'hFF, 8'hFF, 8'hFF, 8'hFF}; // Overflow
	reg[399:0] test4 = {8'd4}; // 1 Termen
	reg[399:0] test5_a = {8'd5, 8'd5}; 
	reg[399:0] test5_b = {8'd10, 8'd112};  // One cc delay
	reg[399:0] test8 = {0}; // Big test
	


	// Instantiate the Design Under Test (DUT)
	adder8 dut (
		.data_out(data_out),
		.valid_out(valid_out),
		.eot_out(eot_out),
		.sot_out(sot_out),
		.error(error),
		
		.data_in(data_in),
		.valid_in(valid_in),
		.eot_in(eot_in),
		.sot_in(sot_in),
		
		.clk(clk),
		.rst_n(rst_n)
	);

	// Generating test5 
	initial begin
		for( i = 0; i < 40; i = i + 1)
			test8[8 * (i+1) - 1 -: 8 ] = i;
	end 

	initial begin
		rst_n 			= 0;
		clk 				= 0;
		data_in_reg		= 0;
		valid_in_reg 	= 0;
		sot_in_reg	 	= 0;
		eot_in_reg		= 0;
		error_reg		= 0;
		
		repeat(10)
			@(posedge clk);
			
		rst_n = 1;
		
		repeat(10)
			@(posedge clk);
		
		driver(test0,9);
		monitor('hFF,0);
		
		driver(test1,9);
		monitor(71,0);
		
		driver(test2,5);
		monitor('h24F,0);
		
		
		driver(test3,5);
		monitor('h3FF,1);
		
		driver(test4,1);
		monitor(4,0);
		
		driver(test5_a,2);
		@(posedge clk);
		driver(test5_b,2);
		monitor('d122,0);
		
		
		driver(test8,40);
		monitor('h30C,0);
		
		$display("============================================");
		$display("Test finished with 0 error!");
		$display("============================================");
		
		 
	end
	
	always #10 clk = !clk;
	
	
	// Assign inputs
	assign data_in 	= data_in_reg;
	assign valid_in 	= valid_in_reg;
	assign sot_in 		= sot_in_reg;
	assign eot_in 		= eot_in_reg;
	
	////////////////////////////////////////////////////
	//////////////DO NOT TOUCH THIS PART////////////////
	////////////////////////////////////////////////////
	
	task driver(input reg[399:0] data, input integer n);
	begin
		valid_in_reg = 1;
		sot_in_reg = 1;
		
		for(i = 0; i < n - 1; i = i + 1)
		begin
			data_in_reg	= data[8 * (i+1) - 1 -: 8 ];
			@(posedge clk);
			if (i == 0) 
				sot_in_reg	 	= 0;

		end
		eot_in_reg		= 1;
		
		data_in_reg = data[8 * n - 1 -: 8];
		
		@(posedge clk);
		
		sot_in_reg	 	= 0;
		data_in_reg		= 0;
		valid_in_reg 	= 0;
		eot_in_reg		= 0;
		
	end
	endtask
	
	////////////////////////////////////////////////////
	//////////////DO NOT TOUCH THIS PART////////////////
	////////////////////////////////////////////////////
	
	
	task monitor(input reg[`AC_ADDER_MAXIMUM_SUM_WIDTH - 1 : 0] expected_data, input reg expected_error);
	begin
		$display("[Test %0d STARTING]: ...",test_index);
		error_reg = 0;
		expecting_eot = 0;
		sum = 0;
		wait (valid_out == 1 && sot_out == 1);
		@(posedge clk);
		sum = data_out;
		error_reg = error;
		expecting_eot = expecting_eot | eot_out;
		@(posedge clk);
		
		while(valid_out == 1)
		begin
		sum = {sum, data_out};
		error_reg = error_reg  & error;
		expecting_eot = expecting_eot | eot_out;
		@(posedge clk);
		end
				
		
		if(expected_error !== error_reg)
		begin
			$display("[Test %0d FAILED]: %0s, but %0s",test_index, expected_error ? "Expected error" : "Did not expect error" , error_reg ? "received error" : "did not receive error");
			$finish;
		end
					
					
		if(expected_data !== sum)
		begin
			$display ("[Test %0d FAILED]: Expected sum = %0d, but received %0d", test_index, expected_data, sum);
			$finish;
		end
		
		if(expecting_eot !== 1)
		begin
			$display("[Test %0d FAILED]: Did not receive EOT", test_index);
			$finish;
		end
		
		$display ("[Test %0d PASSED]: Sum correctly checked! \n Expected sum = %0d, received %0d", test_index, expected_data, sum);
		test_index = test_index + 1;
	end
	endtask
	
endmodule

